@@ -31,7 +31,6 @@ from __future__ import with_statement
 
 import hashlib
 import os.path
-import subprocess
 import urllib2
 import urlparse
 import re
@@ -40,10 +39,12 @@ import sys
 import debian.deb822
 import debian.debian_support
 
+from devscripts.logger import Logger
+
 from ubuntutools.config import UDTConfig
-from ubuntutools.logger import Logger
 from ubuntutools.lp.lpapicache import (Launchpad, Distribution,
                                        SourcePackagePublishingHistory)
+from ubuntutools import subprocess
 
 class DownloadError(Exception):
     "Unable to pull a source package"
@@ -94,16 +95,40 @@ class Dsc(debian.deb822.Dsc):
             return hash_func.hexdigest() == digest
         return False
 
+    def compare_dsc(self, other):
+        """Check whether any files in these two dscs that have the same name
+        also have the same checksum."""
+        for field, key in (('Checksums-Sha256', 'sha256'),
+                           ('Checksums-Sha1', 'sha1'),
+                           ('Files', 'md5sum')):
+            if field not in self or field not in other:
+                continue
+            our_checksums = \
+                dict((entry['name'], (int(entry['size']), entry[key]))
+                     for entry in self[field])
+            their_checksums = \
+                dict((entry['name'], (int(entry['size']), entry[key]))
+                     for entry in other[field])
+            for name, (size, checksum) in our_checksums.iteritems():
+                if name not in their_checksums:
+                    # file only in one dsc
+                    continue
+                if (size != their_checksums[name][0] or
+                    checksum != their_checksums[name][1]):
+                    return False
+            return True # one checksum is good enough
+        return True
+
 
 class SourcePackage(object):
     """Base class for source package downloading.
     Use DebianSourcePackage or UbuntuSourcePackage instead of using this
     directly.
     """
-    distribution = 'unknown'
+    distribution = None
 
     def __init__(self, package=None, version=None, component=None,
-                 dscfile=None, lp=None, mirrors=(), workdir='.'):
+                 dscfile=None, lp=None, mirrors=(), workdir='.', quiet=False):
         "Can be initialised either using package, version or dscfile"
         assert ((package is not None and version is not None)
                 or dscfile is not None)
@@ -111,6 +136,7 @@ class SourcePackage(object):
         self.source = package
         self._lp = lp
         self.workdir = workdir
+        self.quiet = quiet
 
         # Cached values:
         self._component = component
@@ -123,8 +149,9 @@ class SourcePackage(object):
         # Mirrors
         self._dsc_source = dscfile
         self.mirrors = list(mirrors)
-        self.masters = [UDTConfig.defaults['%s_MIRROR'
-                                           % self.distribution.upper()]]
+        if self.distribution:
+            self.masters = [UDTConfig.defaults['%s_MIRROR'
+                                               % self.distribution.upper()]]
         if dscfile is not None:
             if self.source is None:
                 self.source = 'unknown'
@@ -175,9 +202,8 @@ class SourcePackage(object):
     @property
     def dsc(self):
         "Return a the Dsc"
-        if not self._dsc:
-            if self._dsc_fetched:
-                self._dsc = Dsc(file(self.dsc_pathname, 'rb').read())
+        if self._dsc is None:
+            self.pull_dsc()
         return self._dsc
 
     def _mirror_url(self, mirror, filename):
@@ -212,34 +238,35 @@ class SourcePackage(object):
             if parsed.scheme == '':
                 self._dsc_source = 'file://' + os.path.abspath(self._dsc_source)
                 parsed = urlparse.urlparse(self._dsc_source)
-
-            source_is_workdir = (os.path.realpath(os.path.dirname(parsed.path))
-                                 == os.path.realpath(self.workdir))
-            if parsed.scheme == 'file' and source_is_workdir:
-                # Temporarily rename to the filename we are going to open
-                os.rename(parsed.path, self.dsc_pathname)
-            else:
-                if not self._download_file(self._dsc_source, self.dsc_name):
-                    raise DownloadError('dsc not found')
+            url = self._dsc_source
         else:
-            if not self._download_file(self._lp_url(self.dsc_name),
-                                       self.dsc_name):
-                raise DownloadError('dsc not found')
+            url = self._lp_url(self.dsc_name)
+        self._download_dsc(url)
+
         self._check_dsc()
 
+    def _download_dsc(self, url):
+        "Download specified dscfile and parse"
+        # Launchpad will try to redirect us to plain-http Launchpad Librarian,
+        # but we want SSL when fetching the dsc
+        if url.startswith('https') and url.endswith('.dsc'):
+            opener = urllib2.build_opener(ForceHTTPSRedirectHandler())
+        else:
+            opener = urllib2.build_opener()
+        try:
+            f = opener.open(url)
+            self._dsc = Dsc(f.read())
+            f.close()
+        except urllib2.URLError:
+            raise DownloadError('dsc not found')
+        self._dsc_fetched = True
+
     def _check_dsc(self, verify_signature=False):
         "Check that the dsc matches what we are expecting"
-        assert os.path.exists(self.dsc_pathname)
-        self._dsc_fetched = True
-        old_pathname = self.dsc_pathname
-
+        assert self._dsc is not None
         self.source = self.dsc['Source']
         self.version = debian.debian_support.Version(self.dsc['Version'])
 
-        # If source or version was previously unknown
-        if self.dsc_pathname != old_pathname:
-            os.rename(old_pathname, self.dsc_pathname)
-
         valid = False
         message = None
         gpg_info = None
@@ -270,58 +297,62 @@ class SourcePackage(object):
         else:
             Logger.info(message)
 
+    def _write_dsc(self):
+        "Write dsc file to workdir"
+        if self._dsc is None:
+            self.pull_dsc()
+        with open(self.dsc_pathname, 'w') as f:
+            f.write(self.dsc.raw_text)
+
     def _download_file(self, url, filename):
         "Download url to filename in workdir."
-        logurl = url
-        if os.path.basename(url) != filename:
-            logurl += ' -> ' + filename
         pathname = os.path.join(self.workdir, filename)
-        if self.dsc and not url.endswith('.dsc'):
-            if self.dsc.verify_file(pathname):
-                Logger.debug('Using existing %s', filename)
-                return True
-            size = [entry['size'] for entry in self.dsc['Files']
-                    if entry['name'] == filename]
-            assert len(size) == 1
-            size = int(size[0])
-            Logger.normal('Downloading %s (%0.3f MiB)', logurl,
-                          size / 1024.0 / 1024)
-        else:
-            Logger.normal('Downloading %s', logurl)
-
-        # Launchpad will try to redirect us to plain-http Launchpad Librarian,
-        # but we want SSL when fetching the dsc
-        if url.startswith('https') and url.endswith('.dsc'):
-            opener = urllib2.build_opener(ForceHTTPSRedirectHandler())
-        else:
-            opener = urllib2.build_opener()
+        if self.dsc.verify_file(pathname):
+            Logger.debug('Using existing %s', filename)
+            return True
+        size = [entry['size'] for entry in self.dsc['Files']
+                if entry['name'] == filename]
+        assert len(size) == 1
+        size = int(size[0])
+        host = urlparse.urlparse(url).hostname
+        if not self.quiet:
+            Logger.normal('Downloading %s from %s (%0.3f MiB)',
+                          filename, host, size / 1024.0 / 1024)
 
         try:
-            in_ = opener.open(url)
+            in_ = urllib2.build_opener().open(url)
         except urllib2.URLError:
             return False
 
-        with open(pathname, 'wb') as out:
-            while True:
-                block = in_.read(10240)
-                if block == '':
-                    break
-                out.write(block)
-                Logger.stdout.write('.')
+        downloaded = 0
+        bar_width = 60
+        try:
+            with open(pathname, 'wb') as out:
+                while True:
+                    block = in_.read(10240)
+                    if block == '':
+                        break
+                    downloaded += len(block)
+                    out.write(block)
+                    if not self.quiet:
+                        percent = downloaded * 100 // size
+                        bar = '=' * int(round(downloaded * bar_width / size))
+                        bar = (bar + '>' + ' ' * bar_width)[:bar_width]
+                        Logger.stdout.write('[%s] %#3i%%\r' % (bar, percent))
+                        Logger.stdout.flush()
+            in_.close()
+        finally:
+            if not self.quiet:
+                Logger.stdout.write(' ' * (bar_width + 7) + '\r')
                 Logger.stdout.flush()
-        in_.close()
-        Logger.stdout.write(' done\n')
-        Logger.stdout.flush()
-        if self.dsc and not url.endswith('.dsc'):
-            if not self.dsc.verify_file(pathname):
-                Logger.error('Checksum does not match.')
-                return False
+        if not self.dsc.verify_file(pathname):
+            Logger.error('Checksum for %s does not match.', filename)
+            return False
         return True
 
     def pull(self):
         "Pull into workdir"
-        if self.dsc is None:
-            self.pull_dsc()
+        self._write_dsc()
         for entry in self.dsc['Files']:
             name = entry['name']
             for url in self._source_urls(name):
@@ -441,8 +472,11 @@ class DebianSourcePackage(SourcePackage):
         # Not all Debian Source packages get imported to LP
         # (or the importer could be lagging)
         for url in self._source_urls(self.dsc_name):
-            if self._download_file(url, self.dsc_name):
-                break
+            try:
+                self._download_dsc(url)
+            except DownloadError:
+                continue
+            break
         else:
             raise DownloadError('dsc could not be found anywhere')
         self._check_dsc(verify_signature=True)
@@ -518,17 +552,23 @@ def rmadison(url, package, suite=None, arch=None):
     cmd.append(package)
     process = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE, close_fds=True)
-    output = process.communicate()[0]
+    output, error_output = process.communicate()
     try:
         assert process.wait() == 0
     except AssertionError:
-        print "Request failed: install the liburi-perl package and try again."
+        if error_output:
+            Logger.error('rmadison failed with: %s', error_output)
+            sys.exit(1)
+        raise
 
     # rmadison uses some shorthand
     if suite:
         suite = suite.replace('-proposed-updates', '-p-u')
 
+    # pylint bug: http://www.logilab.org/ticket/46273
+    # pylint: disable=E1103
     for line in output.strip().splitlines():
+        # pylint: enable=E1103
         pkg, ver, dist, archs = [x.strip() for x in line.split('|')]
         comp = 'main'
         if '/' in dist:
