@@ -53,7 +53,7 @@ _POCKETS = ('Release', 'Security', 'Updates', 'Proposed', 'Backports')
 class _Launchpad(object):
     '''Singleton for LP API access.'''
 
-    def login(self, service=service):
+    def login(self, service=service, api_version=api_version):
         '''Enforce a non-anonymous login.'''
         if not self.logged_in:
             try:
@@ -201,7 +201,7 @@ class Distribution(BaseWrapper):
             cached = Distribution(Launchpad.distributions[dist])
         return cached
 
-    def getArchive(self, archive = None):
+    def getArchive(self, archive=None):
         '''
         Returns an Archive object for the requested archive.
         Raises a ArchiveNotFoundException if the archive doesn't exist.
@@ -278,7 +278,7 @@ class Archive(BaseWrapper):
         if '_srcpkgs' not in self.__dict__:
             self._srcpkgs = dict()
 
-    def getSourcePackage(self, name, series = None, pocket = 'Release'):
+    def getSourcePackage(self, name, series=None, pocket='Release'):
         '''
         Returns a SourcePackagePublishingHistory object for the most
         recent source package in the distribution 'dist', series and
@@ -307,7 +307,7 @@ class Archive(BaseWrapper):
                 srcpkg = self.getPublishedSources(source_name=name,
                                                   distro_series=series(),
                                                   pocket=pocket,
-                                                  status="Published",
+                                                  status='Published',
                                                   exact_match=True)[0]
                 index = (name, series.name, pocket)
                 self._srcpkgs[index] = SourcePackagePublishingHistory(srcpkg)
@@ -322,6 +322,25 @@ class Archive(BaseWrapper):
 
         return self._srcpkgs[(name, series.name, pocket)]
 
+    def copyPackage(self, source_name, version, from_archive, to_pocket,
+                    to_series=None, include_binaries=False):
+        '''Copy a single named source into this archive.
+
+        Asynchronously copy a specific version of a named source to the
+        destination archive if necessary.  Calls to this method will return
+        immediately if the copy passes basic security checks and the copy
+        will happen sometime later with full checking.
+        '''
+
+        self._lpobject.copyPackage(
+            source_name=source_name,
+            version=version,
+            from_archive=from_archive._lpobject,
+            to_pocket=to_pocket,
+            to_series=to_series,
+            include_binaries=include_binaries
+            )
+
 
 class SourcePackagePublishingHistory(BaseWrapper):
     '''
@@ -417,7 +436,7 @@ class MetaPersonTeam(MetaWrapper):
             except HTTPError, error:
                 if error.response.status == 401:
                     # Anonymous login
-                    cls._me  = None
+                    cls._me = None
                 else:
                     raise
         return cls._me
